%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















%  files.pl
%
%  file handling
%  implements general file-aliasing as in Quintus Prolog

:- ensure_loaded(typeops).

:- module_interface(files).

:- import(prolog_types).
:- import(prolog_meta).
:- import(prolog_io).

:- import(lib).

:- pred compose_filename(filename,filename,filename).
 
:- pred decompose_filename(filename,filename,filename).

:- pred file_search_path(atom,filename).

:- pred file_dealiased(atom,filename).

:- pred find_alias_file(atom,filename,filename).

:- pred open_aliased(filename,io_mode,stream).

:- pred file_exists(filename).

%:- type filename = atom.   % = top
%:- type accessmode = atom. % = top

:- end_interface(files).

:- module_body(files).
%------ 
/*
compose_filename(Dir,FName,DirFName).
 
compose_filename('xdir','ab',X), X='xdir/ab'.
compose_filename('xdir/','ab',X), X='xdir/ab'.
compose_filename('','ab',X), X='ab'.
compose_filename('.','ab',X), X='./ab'.
compose_filename(xdir,'a/b',X), X='xdir/a/b'.
*/

compose_filename('',FName,FName).
compose_filename(Dir,FName,DirFName) :-
        name(Dir,DirCode),
        name(FName,FNameCode),
        (last(0'/,DirCode) ->
            append(DirCode,FNameCode,DirFNameCode)
        ;   append(DirCode,[0'/|FNameCode],DirFNameCode)
        ),
        (   name(DirFName,DirFNameCode)
	;   append(DirFNameCode,[0'.,0'p,0'l],DirFNameCodePL),
            name(DirFName,DirFNameCodePL)
	).

decompose_filename(Filename,Dir,File) :-
	name(Filename,FilenameCode),
	decompose_filename1(FilenameCode,DirCode,[],FileCode,[]),
	name(Dir,DirCode),
	name(File,FileCode).

:- pred decompose_filename1(list(char),list(char),list(char),list(char),list(char)).

decompose_filename1([],Dir1,Dir2,File1,File2) :-
	reverse(Dir2,Dir1),
	reverse(File1,File2),!.

decompose_filename1([0'/|Left],Dir1,Dir2,File1,File2) :-
	!,
	append([0'/|File2],Dir2,NewDir2),
	decompose_filename1(Left,Dir1,NewDir2,File1,[]).

decompose_filename1([Char|Left],Dir1,Dir2,File1,File2) :-
	decompose_filename1(Left,Dir1,Dir2,File1,[Char|File2]).

%------ 
% file_search_path(PathAlias, DirectorySpec)
%   see Quintus built-in hook,  manual p. G-48
% 
/*  e.g.
library_directory(xdir) corresponds to
    file_search_path(library,xdir)
:- dynamic file_search_path/2.
*/


 
%------ 
% file_dealiased(FA,F) :-
%    FA is a possibly aliased file name,
%    F is the corresponding file name with aliasing resolved.
%    if no aliased file is found: F = FA.
%
file_dealiased(FA,F) :-
    compound(FA),
        functor(FA,Alias,1),
        arg(1,FA,FName),
        find_alias_file(Alias,FName,F0),
    !,
    F=F0.
file_dealiased(FA,F) :-
    compound(FA),
    !,  % no aliased files exists
    arg(1,FA,F).
file_dealiased(F,F).

find_alias_file(Alias,File,DirF) :-
    file_search_path(Alias,Dir),
    compose_filename(Dir,File,DirF),     % Dir=a, File=b/c
    file_exists(DirF).

:- pred find_alias_file(atom,filename,filename,filename).
find_alias_file(Alias,File,Path,F) :-
    find_alias_file(Alias,File,DirF),
    decompose_filename(DirF,Path,F).            % Path=a/b, F=c





%------ 
% file_exists(+FileName) :-
% 	Succeed if a file exists, otherwise fail
%
%  Quintus manual H.6-4,  library(files)
%		also file_exists(+FileName,+Permission)


%------
% open_aliased(F,Mode,Stream) :-
%   open file named by F with access mode Mode
%   and return Stream. The file name F may be aliased.
%
/* 
open_aliased(F,Mode,Stream) :- open(F,Mode,Stream). % Quintus
    open/2, open/3  in Quintus already recognize path-aliases 
 
open_aliased(F,Mode,Stream) :-
    file_dealiased(F,F2),
    open(F2,Mode,Stream).
*/

:- end_module(files).
