%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















:- ensure_loaded(typeops).

:- module_interface(lib).
	
%  lib.pl
%
%  a few common library predicates 
%

:- import(prolog_types).
:- import(prolog_num).
:- import(prolog_arith).
:- import(prolog_meta).

%------- 
% This file only contains type declarations with clauses commented out.
% The real implementation of the predicates can be found in 
% various prolog system dependend files, e.g., 
% lib_sisctus.pl lib_quintus.pl ... lib_*.pl
%
% It's awful that there are no standard libraries or header files for Prolog.
%

 
%-------
:- pred append(list(T),list(T),list(@T)).
/*
append([],L,L).
append([H|T],L,[H|TL]) :- append(T,L,TL).
*/


%-------
% det_append/3 is the deterministic variant of append/3
% with mode  append(+,+,-).
%
:- pred det_append(list(T),list(T),list(@T)).
/* det_append is a builtin in BinProlog
det_append(A,B,AB) :- append(A,B,AB). 
*/

:- pred last(T,list(@T)).
% last(X,L) :- append(_,[X],L).
% last(X,L) :- append(_,X,L).
%                       type error

%-------
:- pred reverse(list(@T),list(@T)).
/*
reverse([],[]).
reverse([H|T],R) :- reverse(T,RT),append(RT,[H],R).
*/



%-------
:- pred length(list(_T),nat).


%-------
% same_length(L1,L2) :-
%       lists L1 and L2 have the same length.
%
:- pred same_length(list(_),list(_)).
/*
same_length([],[]).
same_length([_|L1],[_|L2]) :- same_length(L1,L2).
*/

%-------
% remove_duplicates( LD, L ) :-
%       the list L contains every member of list LD without dup's
%
:- pred remove_duplicates(list(@T),list(@T)).
/*
remove_duplicates([],[]).
remove_duplicates([X|Xs],Ys) :-
        member(X,Xs), !,
        remove_duplicates(Xs,Ys).
remove_duplicates([X|Xs],[X|Ys]) :-
        % not member(X,Xs),
        remove_duplicates(Xs,Ys).
*/

%-------
% subset( Set1, Set2) :-
%       every member of Set1 is member of Set2
%
:- pred subset(list(T),list(@T)).
/*
subset([],_).
subset([X|Xs],S) :-
        member(X,S),
        subset(Xs,S).
*/


%-------
:- pred is_list(list(_)).
/*
is_list([]).
is_list([_|_]).
*/


%-------
:- pred member(T,list(@T)).
/*     BinProlog built-in 
member(X,[X|_]).
member(X,[_|T]) :- member(X,T).
*/


%-------
:- pred memberchk(T,list(@T)).
/*
memberchk(Element, [Element|_]) :- !.
memberchk(Element, [_|Rest]) :-
        memberchk(Element, Rest).
*/

%-------
% non_member(X, L) :-
%      true if X ist not in list L
%
:- pred non_member(T,list(@T)).
/*
non_member(_,[]).
non_member(X, [X1|L]) :-
        X \== X1, non_member(X, L).
*/


%-------
%  delete(Old_list, Elem, New_list) :-
%      New_list equals Old_list 
%      except for the removal of all occurrences of Elem.
%
:- pred delete(list(@T),T,list(T)).
/*
delete([], _, []).
delete([Elem1 | Rest_of_old], Elem1, New_list) :-
        !,delete(Rest_of_old, ELem1, New_list).
delete([Elem2 | Rest_of_old], Elem1, [Elem2|Rest_of_new]) :-
        !,delete(Rest_of_old, Elem1, Rest_of_new).
*/


%-------
% subtract(Set1,Set2,Diff)
% Diff consists of all elements from Set1 which are not also in Set2
%
:- pred subtract(list(top),list(top),list(top)).
/*
subtract([], _, []).
subtract([Element|Residue], Set, Difference) :-
        memberchk(Element, Set), !,
        subtract(Residue, Set, Difference).
subtract([Element|Residue], Set, [Element|Difference]) :-
        subtract(Residue, Set, Difference).
*/ 


 
:- pred variant(@T,@T).
/*                      library(terms)
variant(A, B) :-
        subsumes_chk(A,B),
        subsumes_chk(B,A).
*/

:- end_interface(lib).