%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















:- ensure_loaded(typeops).

:- module_body(posat).

%    satisfy constraints over a poset
%

:- import(prolog_types).
%:- import(prolog_meta).
%:- import(prolog_io).

:- import(csppo).
:- import(glib).
:- import(lib).
:- import(checking).
%:- import(poset).  % import abstract type poset(T)


%----
% satisfy_constraints_once(PO,C) :-
%       constraints C are satisfied in poset PO,
%       C may contain logical variables when called.
%	PO must be a quasi-lattice.
%       only the first solution is computed.

satisfy_constraints_once(PO,C) :- 
%write(satisfy),nl,write(C),nl,
	satisfy_constraints(PO,C),!
%,write(C),nl
.
%----
% satisfy_constraints(PO,C) :-
%       constraints C are satisfied in poset PO,
%       C may contain logical variables when called.
%	PO must be a quasi-lattice.
%

satisfy_constraints(PO,C) :- 
        opt_cseq(C,[],C2), satisfy_constraints_b(PO,C2).
        %satisfy_constraints_b(PO,C).  % with non-optimized sequence

:- pred satisfy_constraints_b(poset(@T),list(po_constraint(T))).
satisfy_constraints_b(_PO, []) :- !.
satisfy_constraints_b(PO, [X=Y|C]) :- !,
	X = Y,
        satisfy_constraints_b(PO,C).
satisfy_constraints_b(PO, [X=<Y|C]) :- !,
        po_test_leq(PO, X,Y),
        satisfy_constraints_b(PO,C).
satisfy_constraints_b(PO, [lub(Y,S)|C]) :- !, true,
        lubt_set(PO,S,Y),
        satisfy_constraints_b(PO,C).
satisfy_constraints_b(_,_) :- true.

% local functions

:- pred lubt_set(poset(@T),list(T),T).
lubt_set(_,[X],X).   % X is not nec. a simple term !?
lubt_set(PO,[X,Y|Ys],Z) :-
%!?!?   po_lub(PO, X,Y,Z0),
        po_mub(PO, X,Y,Z0),   % in quasi-lattice lub = mub
        lubt_set(PO, [Z0|Ys],Z).
        
%----
% optimize sequence for constraints
%
% opt_cseq(C,Sel,CS) :-
%       Constraints C, Variables in Sel already selected,
%       CS is optimized sequence
:- pred opt_cseq(list(po_constraint(@T)),list(T),list(po_constraint(T))).

opt_cseq([],_,[]).
opt_cseq(C,Sel,[X=Y|CS]) :-
        member_elim(X=Y,C,C0),
        !,
        opt_cseq(C0,Sel,CS).
opt_cseq(C,Sel,[X=<Y|CS]) :-
        member_elim(X=<Y,C,C0),
        ground(X), ground(Y),
        !,
        opt_cseq(C0,Sel,CS).
opt_cseq(C,Sel,[X=<Y|CS]) :-
        member_elim(X=<Y,C,C0),
        ( ( ground(X), var_member(Y,Sel) ) ;
          ( ground(Y), var_member(X,Sel) ) ),
        !,
        opt_cseq(C0,Sel,CS).
opt_cseq(C,Sel,[X=<Y|CS]) :-
        member_elim(X=<Y,C,C0),
        ( ground(X) ; var_member(X,Sel) ),
        ( ground(Y) ; var_member(Y,Sel) ),
        !,
        opt_cseq(C0,Sel,CS).
opt_cseq(C,Sel,[lub(B,L)|CS]) :-
        member_elim(lub(B,L),C,C0),
        ground(L),
        !,
        opt_cseq(C0,[B|Sel],CS).
opt_cseq(C,Sel,[lub(B,L)|CS]) :-
        member_elim(lub(B,L),C,C0),
        all_known(Sel,L),
        !,
        opt_cseq(C0,[B|Sel],CS).
opt_cseq(C,Sel,[X=<Y|CS]) :-
        member_elim(X=<Y,C,C0),
        ( ground(X) ; var_member(X,Sel) ),
        !,
        opt_cseq(C0,[Y|Sel],CS).
opt_cseq(C,Sel,[X=<Y|CS]) :-
        member_elim(X=<Y,C,C0),
        ( ground(Y) ; var_member(Y,Sel) ),
        !,
        opt_cseq(C0,[X|Sel],CS).
opt_cseq(C,Sel,[lub(B,L)|CS]) :-
        member_elim(lub(B,L),C,C0),
        almostall_known(Sel,[B|L]),
        !,
        opt_cseq(C0,[B|Sel],CS).
opt_cseq([X=<Y|C],Sel,[X=<Y|CS]) :-
        true,
        true,
        !,
        opt_cseq(C,[X,Y|Sel],CS).
opt_cseq([lub(B,L)|C],Sel,[lub(B,L)|CS]) :-
        append([B|L],Sel,Sel2),
        opt_cseq(C,Sel2,CS).
 
:- pred all_known(list(T),list(T)). 
%
all_known(_,[]).
all_known(Sel,[X|Xs]) :-
        (ground(X) ; var_member(X,Sel)),
        all_known(Sel,Xs).
 
:- pred almostall_known(list(T),list(T)). 
%
almostall_known(_,[]).
almostall_known(Sel,[X|Xs]) :-
        (ground(X) ; var_member(X,Sel)),
        !,
        almostall_known(Sel,Xs).
almostall_known(Sel,[_|Xs]) :-
        all_known(Sel,Xs).
 
:- end_module(posat).
